@@ -1,9 +1,9 @@
-;;; evil-repeat.el --- Repeat system
+;;; evil-repeat.el --- Repeat system -*- lexical-binding: t -*-
 
 ;; Author: Frank Fischer <frank.fischer at mathematik.tu-chemnitz.de>
 ;; Maintainer: Vegard Øye <vegard_oye at hotmail.com>
 
-;; Version: 1.2.12
+;; Version: 1.14.2
 
 ;;
 ;; This file is NOT part of GNU Emacs.
@@ -50,9 +50,9 @@
 ;;                             \___/
 ;;
 ;; The recording of a repeat is started in one of two cases: Either a
-;; command is about being executed (in pre-command-hook) or normal
+;; command is about to be executed (in pre-command-hook) or normal
 ;; state is exited. The recording is stopped whenever a command has
-;; being completed and evil is in normal state afterwards. Therefore,
+;; been completed and evil is in normal state afterwards. Therefore,
 ;; a non-inserting command in normal-state is recorded as a single
 ;; repeat unit. In contrast, if the command leaves normal state and
 ;; starts insert-state, all commands that are executed until
@@ -61,9 +61,9 @@
 ;; of all commands that are executed starting and ending in normal
 ;; state.
 ;;
-;; Not all commands are recored. There are several commands that are
+;; Not all commands are recorded. There are several commands that are
 ;; completely ignored and other commands that even abort the currently
-;; active recording, e.g., commands that change the current buffer.
+;; active recording, e.g., commands that switch buffer.
 ;;
 ;; During recording the repeat information is appended to the variable
 ;; `evil-repeat-info', which is cleared when the recording
@@ -276,7 +276,8 @@ has :repeat nil."
         ;; called.
         (evil-repeat-abort))
        ;; ignore those commands completely
-       ((null repeat-type))
+       ((or (null repeat-type)
+            (evil-mouse-events-p (this-command-keys))))
        ;; record command
        (t
         ;; In normal-state or visual state, each command is a single
@@ -320,6 +321,16 @@ invoked the current command"
   (clear-this-command-keys t)
   (setq evil-repeat-keys ""))
 
+(defun evil-this-command-keys (&optional post-cmd)
+  "Version of `this-command-keys' with finer control over prefix args."
+  (let ((arg (if post-cmd current-prefix-arg prefix-arg)))
+    (vconcat
+     (when (and (numberp arg)
+                ;; Only add prefix if no repeat info recorded yet
+                (null evil-repeat-info))
+       (string-to-vector (number-to-string arg)))
+     (this-single-command-keys))))
+
 (defun evil-repeat-keystrokes (flag)
   "Repeation recording function for commands that are repeated by keystrokes."
   (cond
@@ -327,11 +338,11 @@ invoked the current command"
     (when evil-this-register
       (evil-repeat-record
        `(set evil-this-register ,evil-this-register)))
-    (setq evil-repeat-keys (this-command-keys)))
+    (setq evil-repeat-keys (evil-this-command-keys)))
    ((eq flag 'post)
-    (evil-repeat-record (if (zerop (length (this-command-keys)))
+    (evil-repeat-record (if (zerop (length (evil-this-command-keys t)))
                             evil-repeat-keys
-                          (this-command-keys)))
+                          (evil-this-command-keys t)))
     ;; erase commands keys to prevent double recording
     (evil-clear-command-keys))))
 
@@ -388,15 +399,15 @@ If CHANGE is specified, it is added to `evil-repeat-changes'."
 
 (defun evil-repeat-insert-at-point (flag)
   "Repeation recording function for commands that insert text in region.
-This records text insertion when a command inserts some text in a
-buffer between (point) and (mark)."
+For example `mouse-yank-primary'. This records text insertion when a command
+inserts some text in a buffer between (point) and (mark)."
   (cond
    ((eq flag 'pre)
     (add-hook 'after-change-functions #'evil-repeat-insert-at-point-hook nil t))
    ((eq flag 'post)
     (remove-hook 'after-change-functions #'evil-repeat-insert-at-point-hook t))))
 
-(defun evil-repeat-insert-at-point-hook (beg end length)
+(defun evil-repeat-insert-at-point-hook (beg end _length)
   (let ((repeat-type (evil-repeat-type this-command t)))
     (when (and (evil-repeat-recording-p)
                (eq repeat-type 'evil-repeat-insert-at-point)
@@ -542,6 +553,9 @@ and only if COUNT is non-nil."
      (t
       (evil-execute-repeat-info repeat-info)))))
 
+;; Keep the compiler happy - this is a buffer local var
+(defvar evil--execute-normal-return-state)
+
 (evil-define-command evil-repeat (count &optional save-point)
   "Repeat the last editing command with count replaced by COUNT.
 If SAVE-POINT is non-nil, do not move point."
@@ -557,7 +571,8 @@ If SAVE-POINT is non-nil, do not move point."
       (evil-repeat count)))
    (t
     (unwind-protect
-        (let ((confirm-kill-emacs t)
+        (let ((evil-last-find-temp evil-last-find)
+              (confirm-kill-emacs t)
               (kill-buffer-hook
                (cons #'(lambda ()
                          (user-error "Cannot delete buffer in repeat command"))
@@ -566,8 +581,11 @@ If SAVE-POINT is non-nil, do not move point."
           (evil-with-single-undo
             (setq evil-last-repeat (list (point) count undo-pointer))
             (evil-execute-repeat-info-with-count
-             count (ring-ref evil-repeat-ring 0))))
-      (evil-normal-state)))))
+             count (ring-ref evil-repeat-ring 0))
+            (setq evil-last-find evil-last-find-temp)))
+      (if (eq 'evil-execute-in-normal-state last-command)
+          (evil-change-state evil--execute-normal-return-state)
+        (evil-normal-state))))))
 
 ;; TODO: the same issue concering disabled undos as for `evil-paste-pop'
 (evil-define-command evil-repeat-pop (count &optional save-point)
